@@ -45,7 +45,7 @@ AthenaConnection <-
     
     s3_staging_dir <- s3_staging_dir %||% get_aws_env("AWS_ATHENA_S3_STAGING_DIR")
     
-    if(is.null(s3_staging_dir)) {stop("Please set `s3_staging_dir` either in parameter `s3_staging_dir`, evnironmental varaible `AWS_ATHENA_S3_STAGING_DIR`",
+    if(is.null(s3_staging_dir)) {stop("Please set `s3_staging_dir` either in parameter `s3_staging_dir`, environmental varaible `AWS_ATHENA_S3_STAGING_DIR`",
                                       "or when work_group is defined in `create_work_group()`", call. = F)}
     
     
@@ -54,7 +54,7 @@ AthenaConnection <-
                  dbms.name = schema_name, work_group = work_group,
                  poll_interval = poll_interval, encryption_option = encryption_option,
                  kms_key = kms_key, expiration = aws_expiration)
-
+    
     res <- new("AthenaConnection",  ptr = ptr, info = info, quote = "`")
   }
 
@@ -110,7 +110,9 @@ setMethod(
   function(conn, ...) {
     if (!dbIsValid(conn)) {
       warning("Connection already closed.", call. = FALSE)
-    } else {eval.parent(substitute(conn@ptr <- NULL))}
+    } else {
+      on_connection_closed(conn)
+      eval.parent(substitute(conn@ptr <- NULL))}
     invisible(NULL)
   })
 
@@ -314,6 +316,8 @@ setMethod(
 #' Returns the unquoted names of Athena tables accessible through this connection.
 #' @name dbListTables
 #' @inheritParams DBI::dbListTables
+#' @param schema Athena schema, default set to NULL to return all tables from all Athena schemas.
+#'               Note: The use of DATABASE and SCHEMA is interchangeable within Athena.
 #' @aliases dbListTables
 #' @return \code{dbListTables()} returns a character vector with all the tables from Athena.
 #' @seealso \code{\link[DBI]{dbListTables}}
@@ -340,17 +344,68 @@ NULL
 #' @export
 setMethod(
   "dbListTables", "AthenaConnection",
-  function(conn,...){
+  function(conn, schema = NULL,...){
     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
     glue <- conn@ptr$client("glue")
-    tryCatch(Databases <- sapply(glue$get_databases()$DatabaseList,function(x) x$Name),
-             error = function(e) py_error(e))
-    tryCatch(output <- lapply(Databases, function (x) glue$get_tables(DatabaseName = x)$TableList),
+    
+    if(is.null(schema)){
+      tryCatch(schema <- sapply(glue$get_databases()$DatabaseList,function(x) x$Name),
+               error = function(e) py_error(e))}
+    tryCatch(output <- lapply(schema, function (x) glue$get_tables(DatabaseName = x)$TableList),
              error = function(e) py_error(e))
     unlist(lapply(output, function(x) sapply(x, function(y) y$Name)))
   }
 )
 
+#' List Athena Schema, Tables and TableTypes
+#'
+#' Method to get Athena schema, tables and table types return as a data.frame
+#' @name dbGetTables
+#' @inheritParams DBI::dbListTables
+#' @param schema Athena schema, default set to NULL to return all tables from all Athena schemas.
+#'               Note: The use of DATABASE and SCHEMA is interchangeable within Athena.
+#' @aliases dbGetTables
+#' @return \code{dbGetTables()} returns a data.frame.
+#' @examples 
+#' \dontrun{
+#' # Note: 
+#' # - Require AWS Account to run below example.
+#' # - Different connection methods can be used please see `noctua::dbConnect` documnentation
+#' 
+#' library(DBI)
+#' library(RAthena)
+#' 
+#' # Demo connection to Athena using profile name 
+#' con <- dbConnect(RAthena::athena())
+#'              
+#' # Return hierarchy of tables in Athena
+#' dbGetTables(con)
+#' 
+#' # Disconnect conenction
+#' dbDisconnect(con)
+#' }
+NULL
+
+#' @rdname dbGetTables
+#' @export
+setGeneric("dbGetTables", function(conn, ...) standardGeneric("dbGetTables"))
+
+#' @rdname dbGetTables
+#' @export
+setMethod("dbGetTables", "AthenaConnection",
+  function(conn, schema = NULL, ...){
+    if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
+    glue <- conn@ptr$client("glue")
+  if(is.null(schema)){
+    tryCatch(schema <- sapply(glue$get_databases()$DatabaseList,function(x) x$Name),
+             error = function(e) py_error(e))}
+  tryCatch(output <- lapply(schema, function (x) glue$get_tables(DatabaseName = x)$TableList),
+           error = function(e) py_error(e))
+  rbindlist(lapply(output, function(x) rbindlist(lapply(x, function(y) data.frame(Schema = y$DatabaseName,
+                                                                                  TableName=y$Name,
+                                                                                  TableType = y$TableType)))))
+})
+
 
 #' List Field names of Athena table
 #'
@@ -443,13 +498,13 @@ setMethod(
   "dbExistsTable", c("AthenaConnection", "character"),
   function(conn, name, ...) {
     if (!dbIsValid(conn)) {stop("Connection already closed.", call. = FALSE)}
-  
+    
     if(grepl("\\.", name)){
       dbms.name <- gsub("\\..*", "" , name)
       Table <- gsub(".*\\.", "" , name)
     } else {dbms.name <- conn@info$dbms.name
     Table <- tolower(name)}
-
+    
     glue <- conn@ptr$client("glue")
     tryerror <- try(tryCatch(glue$get_table(DatabaseName = dbms.name, Name = Table),
                              error = function(e) py_error(e)), silent = TRUE)
@@ -510,9 +565,13 @@ setMethod(
       dbms.name <- conn@info$dbms.name
       Table <- name}
     
-    if (delete_data) {
-      glue <- conn@ptr$client("glue")
-      s3 <- conn@ptr$resource("s3")
+    glue <- conn@ptr$client("glue")
+    s3 <- conn@ptr$resource("s3")
+    
+    tryCatch(TableType <- glue$get_table(DatabaseName = dbms.name, Name = Table)$Table$TableType,
+             error = function(e) py_error(e))
+    
+    if (delete_data && TableType == "EXTERNAL_TABLE") {
       tryCatch(
         s3_path <- split_s3_uri(glue$get_table(DatabaseName = dbms.name,
                                                Name = Table)$Table$StorageDescriptor$Location),
@@ -536,6 +595,7 @@ setMethod(
     res <- dbExecute(conn, paste("DROP TABLE ", paste(dbms.name, Table, sep = "."), ";"))
     dbClearResult(res)
     if (!delete_data) message("Info: Only Athena table has been removed.")
+    on_connection_updated(conn, Table)
     invisible(TRUE)
   })
 
